@@ -165,7 +165,7 @@
 (defvar *--load-list-file-extension-whitelist* '("lisp" "lsp" "cl" "asd")
   "White list of file extensions allowed with the --load option.")
 
-(defun load-extra-transformation-functions (filename)
+(defun load-extra-transformation-functions (filename &optional verbose)
   "Load an extra filename to tweak pgloader's behavior."
   (let ((pathname (uiop:parse-native-namestring filename)))
     (unless (member (pathname-type pathname)
@@ -173,8 +173,8 @@
                     :test #'string=)
       (error "Unknown lisp file extension: ~s" (pathname-type pathname)))
 
-    (log-message :info "Loading code from ~s" pathname)
-    (load (compile-file pathname :verbose nil :print nil))))
+    (format t "Loading code from ~s~%" pathname)
+    (load (compile-file pathname :verbose verbose :print verbose))))
 
 (defun main (argv)
   "Entry point when building an executable image with buildapp"
@@ -266,6 +266,17 @@
         ;; Should we stop at first error?
         (setf *on-error-stop* on-error-stop)
 
+        ;; load extra lisp code provided for by the user
+        (when load
+          (loop :for filename :in load :do
+             (handler-case
+                 (load-extra-transformation-functions filename debug)
+               ((or simple-condition serious-condition) (e)
+                 (format *error-output*
+                         "Failed to load lisp source file ~s~%" filename)
+                 (format *error-output* "~a~%~%" e)
+                 (uiop:quit +os-code-error+)))))
+
 	;; Now process the arguments
 	(when arguments
 	  ;; Start the logs system
@@ -279,29 +290,20 @@
                 ;; The handler-bind below is to be able to offer a
                 ;; meaningful backtrace to the user in case of unexpected
                 ;; conditions being signaled.
-                (with-monitor ()
-                  ;; tell the user where to look for interesting things
-                  (log-message :log "Main logs in '~a'"
-                               (uiop:native-namestring *log-filename*))
-                  (log-message :log "Data errors in '~a'~%" *root-dir*)
+                (handler-bind
+                    (((and condition (not (or cli-parsing-error
+                                              source-definition-error)))
+                      #'(lambda (condition)
+                          (format *error-output* "KABOOM!~%")
+                          (format *error-output* "FATAL error: ~a~%~a~%~%"
+                                  condition
+                                  (print-backtrace condition debug)))))
 
-                  ;; load extra lisp code provided for by the user
-                  (when load
-                    (loop for filename in load do
-                         (handler-case
-                             (load-extra-transformation-functions filename)
-                           (condition (e)
-                             (log-message :fatal
-                                          "Failed to load lisp source file ~s~%"
-                                          filename)
-                             (log-message :error "~a" e)
-                             (uiop:quit +os-code-error+)))))
-
-                  (handler-bind
-                      ((condition
-                        #'(lambda (condition)
-                            (log-message :fatal "~a"
-                                         (print-backtrace condition debug)))))
+                  (with-monitor ()
+                    ;; tell the user where to look for interesting things
+                    (log-message :log "Main logs in '~a'"
+                                 (uiop:native-namestring *log-filename*))
+                    (log-message :log "Data errors in '~a'~%" *root-dir*)
 
                     (cond
                       ((and regress (= 1 (length arguments)))
@@ -343,14 +345,20 @@
                          (unless (remove-if #'null (mapcar #'second cli-options))
                            (process-command-file arguments)))))))
 
-              (source-definition-error (c)
-                (declare (ignore c))    ; handler-bind printed it out
-                (sleep 0.3)             ; wait until monitor stops...
+              ((or cli-parsing-error source-definition-error) (c)
+                (format *error-output* "~%~a~%~%" c)
+                (let ((lp:*kernel* *monitoring-kernel*))
+                  (lp:end-kernel :wait t))
                 (uiop:quit +os-code-error-bad-source+))
 
               (condition (c)
-                (declare (ignore c))    ; handler-bind printed it out
-                (sleep 0.3)             ; wait until monitor stops...
+                (format *error-output* "~%What I am doing here?~%~%")
+                (format *error-output* "~a~%~%" c)
+                ;; wait until monitor stops...
+                (format *error-output*
+                        "~%Waiting for the monitor thread to complete.~%~%")
+                (let ((lp:*kernel* *monitoring-kernel*))
+                  (lp:end-kernel :wait t))
                 (uiop:quit +os-code-error+)))))
 
         ;; done.
@@ -382,53 +390,104 @@
      :finally (when not-found-list
                 (error 'load-files-not-found-error :filename-list not-found-list))))
 
-(defun process-source-and-target (source target
+(define-condition cli-parsing-error (error) ()
+  (:report (lambda (err stream)
+             (declare (ignore err))
+             (format stream "Could not parse the command line: see above."))))
+
+(defun process-source-and-target (source-string target-string
                                   type encoding set with field cast
                                   before after)
-  "Given exactly 2 CLI arguments, process them as source and target URIs."
-  (let* ((type       (parse-cli-type type))
-         (source-uri (if type
-                         (parse-source-string-for-type type source)
-                         (parse-source-string source)))
-         (type       (when (and source
+  "Given exactly 2 CLI arguments, process them as source and target URIs.
+Parameters here are meant to be already parsed, see parse-cli-optargs."
+  (let* ((type       (handler-case
+                         (parse-cli-type type)
+                       (condition (e)
+                         (log-message :warning
+                                      "Could not parse --type ~s: ~a"
+                                      type e))))
+         (source-uri (handler-case
+                         (if type
+                             (parse-source-string-for-type type source-string)
+                             (parse-source-string source-string))
+                       (condition (e)
+                         (log-message :warning
+                                      "Could not parse source string ~s: ~a"
+                                      source-string e))))
+         (type       (when (and source-string
                                 (typep source-uri 'connection))
                        (parse-cli-type (conn-type source-uri))))
-         (target-uri (ignore-errors (parse-target-string target))))
+         (target-uri (handler-case
+                         (parse-target-string target-string)
+                       (condition (e)
+                         (log-message :error
+                                      "Could not parse target string ~s: ~a"
+                                      target-string e)))))
 
     ;; some verbosity about the parsing "magic"
-    (log-message :info "SOURCE: ~s" source)
-    (log-message :info "TARGET: ~s" target)
+    (log-message :info "    SOURCE: ~s" source-string)
+    (log-message :info "SOURCE URI: ~s" source-uri)
+    (log-message :info "    TARGET: ~s" target-string)
+    (log-message :info "TARGET URI: ~s" target-uri)
 
-    (cond ((and (null source-uri)
-                (null target-uri)
-                ;; (probe-file (uiop:parse-unix-namestring source))
-                ;; (probe-file (uiop:parse-unix-namestring target))
-                )
-           (process-command-file (list source target)))
+    (cond ((and (null source-uri) (null target-uri))
+           (process-command-file (list source-string target-string)))
 
-          ((null source)
+          ((or (null source-string) (null source-uri))
            (log-message :fatal
-                        "Failed to parse ~s as a source URI." source)
+                        "Failed to parse ~s as a source URI." source-string)
            (log-message :log "You might need to use --type."))
 
-          ((null target)
+          ((or (null target-string) (null target-uri))
            (log-message :fatal
                         "Failed to parse ~s as a PostgreSQL database URI."
-                        target)))
+                        target-string)))
 
-    ;; so, we actually have all the specs for the
-    ;; job on the command line now.
-    (when (and source-uri target-uri)
-      (load-data :from source-uri
-                 :into target-uri
-                 :encoding (parse-cli-encoding encoding)
-                 :options  (parse-cli-options type with)
-                 :gucs     (parse-cli-gucs set)
-                 :fields   (parse-cli-fields type field)
-                 :casts    (parse-cli-casts cast)
-                 :before   (parse-sql-file before)
-                 :after    (parse-sql-file after)
-                 :start-logger nil))))
+    (let* ((nb-errors 0)
+           (options (handler-case
+                        (parse-cli-options type with)
+                      (condition (e)
+                        (incf nb-errors)
+                        (log-message :error "Could not parse --with ~s:" with)
+                        (log-message :error "~a" e))))
+           (fields  (handler-case
+                        (parse-cli-fields type field)
+                      (condition (e)
+                        (incf nb-errors)
+                        (log-message :error "Could not parse --fields ~s:" field)
+                        (log-message :error "~a" e)))))
+
+      (destructuring-bind (&key encoding gucs casts before after)
+          (loop :for (keyword option user-string parse-fn)
+             :in `((:encoding  "--encoding" ,encoding ,#'parse-cli-encoding)
+                   (:gucs      "--set"      ,set      ,#'parse-cli-gucs)
+                   (:casts     "--cast"     ,cast     ,#'parse-cli-casts)
+                   (:before    "--before"   ,before   ,#'parse-sql-file)
+                   (:after     "--after"    ,after    ,#'parse-sql-file))
+             :append (list keyword
+                           (handler-case
+                               (funcall parse-fn user-string)
+                             (condition (e)
+                               (incf nb-errors)
+                               (log-message :error "Could not parse ~a ~s: ~a"
+                                            option user-string e)))))
+
+        (unless (= 0 nb-errors)
+          (error 'cli-parsing-error))
+
+        ;; so, we actually have all the specs for the
+        ;; job on the command line now.
+        (when (and source-uri target-uri (= 0 nb-errors))
+          (load-data :from source-uri
+                     :into target-uri
+                     :encoding encoding
+                     :options  options
+                     :gucs     gucs
+                     :fields   fields
+                     :casts    casts
+                     :before   before
+                     :after    after
+                     :start-logger nil))))))
 
 
 ;;;
