@@ -103,11 +103,12 @@
     (bind (((_ _ _ escape-mode) term))
       (cons :escape-mode escape-mode))))
 
-(defrule csv-option (or option-workers
+(defrule csv-option (or option-on-error-stop
+                        option-workers
                         option-concurrency
                         option-batch-rows
                         option-batch-size
-                        option-batch-concurrency
+                        option-prefetch-rows
                         option-max-parallel-create-index
                         option-truncate
                         option-disable-triggers
@@ -414,7 +415,8 @@
        (progn
          ,(sql-code-block pg-db-conn :pre before "before load")
 
-         (let ((truncate                  (getf ',options :truncate))
+         (let ((on-error-stop             (getf ',options :on-error-stop))
+               (truncate                  (getf ',options :truncate))
                (disable-triggers          (getf ',options :disable-triggers))
                (drop-indexes              (getf ',options :drop-indexes))
                (max-parallel-create-index (getf ',options :max-parallel-create-index))
@@ -428,7 +430,8 @@
                                :fields    ',fields
                                :columns   ',columns
                                ,@(remove-batch-control-option
-                                  options :extras '(:worker-count
+                                  options :extras '(:on-error-stop
+                                                    :worker-count
                                                     :concurrency
                                                     :truncate
                                                     :drop-indexes
@@ -439,6 +442,7 @@
                                                 (list :worker-count worker-count))
                                            ,@ (when concurrency
                                                 (list :concurrency concurrency))
+                                           :on-error-stop on-error-stop
                                            :truncate truncate
                                            :drop-indexes drop-indexes
                                            :disable-triggers disable-triggers
