@@ -43,11 +43,12 @@
   (:lambda (source)
     (bind (((_ field-defs _) source)) field-defs)))
 
-(defrule fixed-option (or option-workers
+(defrule fixed-option (or option-on-error-stop
+                          option-workers
                           option-concurrency
                           option-batch-rows
                           option-batch-size
-                          option-batch-concurrency
+                          option-prefetch-rows
                           option-max-parallel-create-index
                           option-truncate
                           option-drop-indexes
@@ -128,7 +129,8 @@
        (progn
          ,(sql-code-block pg-db-conn :pre before "before load")
 
-         (let ((truncate ,(getf options :truncate))
+         (let ((on-error-stop                 ,(getf options :on-error-stop))
+               (truncate                      ,(getf options :truncate))
                (disable-triggers              ,(getf options :disable-triggers))
                (drop-indexes                  ,(getf options :drop-indexes))
                (max-parallel-create-index     ,(getf options :max-parallel-create-index))
@@ -148,6 +150,7 @@
                                                 (list :worker-count worker-count))
                                            ,@ (when concurrency
                                                 (list :concurrency concurrency))
+                                           :on-error-stop on-error-stop
                                            :truncate truncate
                                            :drop-indexes drop-indexes
                                            :disable-triggers disable-triggers
