@@ -18,11 +18,12 @@
   (:lambda (tzopt)
     (bind (((_ tz) tzopt)) (cons :timezone tz))))
 
-(defrule ixf-option (or option-workers
+(defrule ixf-option (or option-on-error-stop
+                        option-workers
                         option-concurrency
                         option-batch-rows
                         option-batch-size
-                        option-batch-concurrency
+                        option-prefetch-rows
                         option-truncate
                         option-disable-triggers
                         option-identifiers-case
