@@ -7,13 +7,18 @@
 ;;;
 ;;; MySQL options
 ;;;
-(defrule mysql-option (or option-workers
+(defrule mysql-option (or option-on-error-stop
+                          option-workers
                           option-concurrency
                           option-batch-rows
                           option-batch-size
-                          option-batch-concurrency
+                          option-prefetch-rows
                           option-max-parallel-create-index
-			  option-truncate
+                          option-single-reader
+                          option-multiple-readers
+                          option-rows-per-range
+			  option-reindex
+                          option-truncate
                           option-disable-triggers
 			  option-data-only
 			  option-schema-only
@@ -60,14 +65,23 @@
   (:lambda (tables)
     (cons :decoding tables)))
 
+
+;;;
+;;; MySQL SET parameters, because sometimes we need that
+;;;
+(defrule mysql-gucs (and kw-set kw-mysql kw-parameters generic-option-list)
+  (:lambda (mygucs) (cons :mysql-gucs (fourth mygucs))))
+
 
 ;;;
 ;;; Allow clauses to appear in any order
 ;;;
 (defrule load-mysql-optional-clauses (* (or mysql-options
+                                            mysql-gucs
                                             gucs
                                             casts
                                             alter-table
+                                            alter-schema
                                             materialize-views
                                             including-matching
                                             excluding-matching
@@ -138,8 +152,9 @@
 
 (defun lisp-code-for-loading-from-mysql (my-db-conn pg-db-conn
                                          &key
-                                           gucs casts views before after options
-                                           alter-table
+                                           gucs mysql-gucs
+                                           casts views before after options
+                                           alter-table alter-schema
                                            ((:including incl))
                                            ((:excluding excl))
                                            ((:decoding decoding-as)))
@@ -147,6 +162,7 @@
      (let* ((*default-cast-rules* ',*mysql-default-cast-rules*)
             (*cast-rules*         ',casts)
             (*decoding-as*        ',decoding-as)
+            (*mysql-settings*     ',mysql-gucs)
             ,@(pgsql-connection-bindings pg-db-conn gucs)
             ,@(batch-control-bindings options)
             ,@(identifier-case-binding options)
@@ -162,6 +178,7 @@
                                      :excluding ',excl
                                      :materialize-views ',views
                                      :alter-table ',alter-table
+                                     :alter-schema ',alter-schema
                                      :set-table-oids t
                                      ,@(remove-batch-control-option options))
 
@@ -172,20 +189,23 @@
     (destructuring-bind (my-db-uri
                          pg-db-uri
                          &key
-                         gucs casts views before after
-                         options alter-table including excluding decoding)
+                         gucs mysql-gucs casts views before after options
+                         alter-table alter-schema
+                         including excluding decoding)
         source
       (cond (*dry-run*
              (lisp-code-for-mysql-dry-run my-db-uri pg-db-uri))
             (t
              (lisp-code-for-loading-from-mysql my-db-uri pg-db-uri
                                                :gucs gucs
+                                               :mysql-gucs mysql-gucs
                                                :casts casts
                                                :views views
                                                :before before
                                                :after after
                                                :options options
                                                :alter-table alter-table
+                                               :alter-schema alter-schema
                                                :including including
                                                :excluding excluding
                                                :decoding decoding))))))
