@@ -8,7 +8,8 @@
                             &key table source-catalog including excluding)
   "Fetch PostgreSQL catalogs for the target database. A PostgreSQL
    connection must be opened."
-  (let* ((catalog   (make-catalog :name dbname))
+  (let* ((*identifier-case* :quote)
+         (catalog   (make-catalog :name dbname))
          (including (cond ((and table (not including))
                            (make-including-expr-from-table table))
 
@@ -45,7 +46,7 @@
                             :sum (length (index-fk-deps index)))))
 
     (when (and table (/= 1 (count-tables catalog)))
-      (error "pgloader found ~d target tables for name ~s|:~{~%  ~a~}"
+      (error "pgloader found ~d target tables for name ~a:~{~%  ~a~}"
              (count-tables catalog)
              (format-table-name table)
              (mapcar #'format-table-name (table-list catalog))))
@@ -88,7 +89,7 @@
   "Return an expression suitable to be used as an :including parameter."
   (let ((schema (or (table-schema table)
                     (query-table-schema table))))
-    (list (cons (schema-name schema)
+    (list (cons (ensure-unquoted (schema-name schema))
                 (list
                  (format-table-name-as-including-exp table))))))
 
@@ -97,26 +98,14 @@
   (let ((table-name (table-name table)))
     (format nil "^~a$" (ensure-unquoted table-name))))
 
-(defun ensure-unquoted (identifier)
-  (cond ((pgloader.quoting::quoted-p identifier)
-         ;; when the table name comes from the user (e.g. in the
-         ;; load file) then we might have to unquote it: the
-         ;; PostgreSQL catalogs does not store object names in
-         ;; their quoted form.
-         (subseq identifier 1 (1- (length identifier))))
-
-        (t identifier)))
-
 (defun query-table-schema (table)
   "Get PostgreSQL schema name where to locate TABLE-NAME by following the
   current search_path rules. A PostgreSQL connection must be opened."
   (make-schema :name
-               (pomo:query (format nil "
-  select nspname
-    from pg_namespace n
-    join pg_class c on n.oid = c.relnamespace
-   where c.oid = '~a'::regclass;"
-                                   (table-name table)) :single)))
+               (pomo:query (format nil
+                                   (sql "/pgsql/query-table-schema.sql")
+                                   (table-name table))
+                           :single)))
 
 
 (defvar *table-type* '((:table    . "r")
@@ -149,36 +138,20 @@
   "Get the list of PostgreSQL column names per table."
   (loop :for (schema-name table-name table-oid name type typmod notnull default)
      :in
-     (pomo:query (format nil "
-    select nspname, relname, c.oid, attname,
-           t.oid::regtype as type,
-           case when atttypmod > 0 then atttypmod - 4 else null end as typmod,
-           attnotnull,
-           case when atthasdef then def.adsrc end as default
-      from pg_class c
-           join pg_namespace n on n.oid = c.relnamespace
-           left join pg_attribute a on c.oid = a.attrelid
-           join pg_type t on t.oid = a.atttypid and attnum > 0
-           left join pg_attrdef def on a.attrelid = def.adrelid
-                                   and a.attnum = def.adnum
-
-     where nspname !~~ '^pg_' and n.nspname <> 'information_schema'
-           and relkind = '~a'
-           ~:[~*~;and (~{~a~^~&~10t or ~})~]
-           ~:[~*~;and (~{~a~^~&~10t and ~})~]
-
-  order by nspname, relname, attnum"
-                         table-type-name
-                         including      ; do we print the clause?
-                         (filter-list-to-where-clause including
-                                                      nil
-                                                      "n.nspname"
-                                                      "c.relname")
-                         excluding      ; do we print the clause?
-                         (filter-list-to-where-clause excluding
-                                                      nil
-                                                      "n.nspname"
-                                                      "c.relname")))
+     (query nil
+            (format nil
+                    (sql "/pgsql/list-all-columns.sql")
+                    table-type-name
+                    including           ; do we print the clause?
+                    (filter-list-to-where-clause including
+                                                 nil
+                                                 "n.nspname"
+                                                 "c.relname")
+                    excluding           ; do we print the clause?
+                    (filter-list-to-where-clause excluding
+                                                 nil
+                                                 "n.nspname"
+                                                 "c.relname")))
      :do
      (let* ((schema    (maybe-add-schema catalog schema-name))
             (table     (maybe-add-table schema table-name :oid table-oid))
@@ -196,40 +169,19 @@
      :for (schema-name name oid
                        table-schema table-name
                        primary unique sql conname condef)
-     :in (pomo:query (format nil "
-  select n.nspname,
-         i.relname,
-         i.oid,
-         rn.nspname,
-         r.relname,
-         indisprimary,
-         indisunique,
-         pg_get_indexdef(indexrelid),
-         c.conname,
-         pg_get_constraintdef(c.oid)
-    from pg_index x
-         join pg_class i ON i.oid = x.indexrelid
-         join pg_class r ON r.oid = x.indrelid
-         join pg_namespace n ON n.oid = i.relnamespace
-         join pg_namespace rn ON rn.oid = r.relnamespace
-         left join pg_constraint c ON c.conindid = i.oid
-                                  and c.conrelid = r.oid
-                                  -- filter out self-fkeys
-                                  and c.confrelid <> r.oid
-   where n.nspname !~~ '^pg_' and n.nspname <> 'information_schema'
-         ~:[~*~;and (~{~a~^~&~10t or ~})~]
-         ~:[~*~;and (~{~a~^~&~10t and ~})~]
-order by n.nspname, r.relname"
-                             including  ; do we print the clause?
-                             (filter-list-to-where-clause including
-                                                          nil
-                                                          "rn.nspname"
-                                                          "r.relname")
-                             excluding  ; do we print the clause?
-                             (filter-list-to-where-clause excluding
-                                                          nil
-                                                          "rn.nspname"
-                                                          "r.relname")))
+     :in (query nil
+                (format nil
+                        (sql "/pgsql/list-all-indexes.sql")
+                        including       ; do we print the clause?
+                        (filter-list-to-where-clause including
+                                                     nil
+                                                     "rn.nspname"
+                                                     "r.relname")
+                        excluding       ; do we print the clause?
+                        (filter-list-to-where-clause excluding
+                                                     nil
+                                                     "rn.nspname"
+                                                     "r.relname")))
      :do (let* ((schema   (find-schema catalog schema-name))
                 (tschema  (find-schema catalog table-schema))
                 (table    (find-table tschema table-name))
@@ -254,54 +206,29 @@ order by n.nspname, r.relname"
                        conoid conname condef
                        cols fcols
                        updrule delrule mrule deferrable deferred)
-     :in
-     (pomo:query (format nil "
- select n.nspname, c.relname, nf.nspname, cf.relname as frelname,
-        r.oid, conname,
-        pg_catalog.pg_get_constraintdef(r.oid, true) as condef,
-        (select string_agg(attname, ',')
-           from pg_attribute
-          where attrelid = r.conrelid and array[attnum] <@ conkey
-        ) as conkey,
-        (select string_agg(attname, ',')
-           from pg_attribute
-          where attrelid = r.confrelid and array[attnum] <@ confkey
-        ) as confkey,
-        confupdtype, confdeltype, confmatchtype,
-        condeferrable, condeferred
-   from pg_catalog.pg_constraint r
-        JOIN pg_class c on r.conrelid = c.oid
-        JOIN pg_namespace n on c.relnamespace = n.oid
-        JOIN pg_class cf on r.confrelid = cf.oid
-        JOIN pg_namespace nf on cf.relnamespace = nf.oid
-   where r.contype = 'f'
-         AND c.relkind = 'r' and cf.relkind = 'r'
-         AND n.nspname !~~ '^pg_' and n.nspname <> 'information_schema'
-         AND nf.nspname !~~ '^pg_' and nf.nspname <> 'information_schema'
-         ~:[~*~;and (~{~a~^~&~10t or ~})~]
-         ~:[~*~;and (~{~a~^~&~10t and ~})~]
-         ~:[~*~;and (~{~a~^~&~10t or ~})~]
-         ~:[~*~;and (~{~a~^~&~10t and ~})~]"
-                         including      ; do we print the clause (table)?
-                         (filter-list-to-where-clause including
-                                                      nil
-                                                      "n.nspname"
-                                                      "c.relname")
-                         excluding      ; do we print the clause (table)?
-                         (filter-list-to-where-clause excluding
-                                                      nil
-                                                      "n.nspname"
-                                                      "c.relname")
-                         including      ; do we print the clause (ftable)?
-                         (filter-list-to-where-clause including
-                                                      nil
-                                                      "nf.nspname"
-                                                      "cf.relname")
-                         excluding      ; do we print the clause (ftable)?
-                         (filter-list-to-where-clause excluding
-                                                      nil
-                                                      "nf.nspname"
-                                                      "cf.relname")))
+     :in (query nil
+                (format nil
+                        (sql "/pgsql/list-all-fkeys.sql")
+                        including       ; do we print the clause (table)?
+                        (filter-list-to-where-clause including
+                                                     nil
+                                                     "n.nspname"
+                                                     "c.relname")
+                        excluding       ; do we print the clause (table)?
+                        (filter-list-to-where-clause excluding
+                                                     nil
+                                                     "n.nspname"
+                                                     "c.relname")
+                        including       ; do we print the clause (ftable)?
+                        (filter-list-to-where-clause including
+                                                     nil
+                                                     "nf.nspname"
+                                                     "cf.relname")
+                        excluding       ; do we print the clause (ftable)?
+                        (filter-list-to-where-clause excluding
+                                                     nil
+                                                     "nf.nspname"
+                                                     "cf.relname")))
      :do (flet ((pg-fk-rule-to-action (rule)
                   (case rule
                     (#\a "NO ACTION")
@@ -353,33 +280,11 @@ order by n.nspname, r.relname"
     (when pkey-oid-list
       (loop :for (schema-name table-name fschema-name ftable-name
                               conoid conname condef index-oid)
-         :in (pomo:query (format nil "
-with pkeys(oid) as (
-  values~{(~d)~^,~}
-),
-     knownfkeys(oid) as (
-  values~{(~d)~^,~}
-),
-  pkdeps as (
-  select pkeys.oid, pg_depend.objid
-    from pg_depend
-         join pkeys on pg_depend.refobjid = pkeys.oid
-   where     classid = 'pg_catalog.pg_constraint'::regclass
-         and refclassid = 'pg_catalog.pg_class'::regclass
-)
- select n.nspname, c.relname, nf.nspname, cf.relname as frelname,
-        r.oid as conoid, conname,
-        pg_catalog.pg_get_constraintdef(r.oid, true) as condef,
-        pkdeps.oid as index_oid
-   from pg_catalog.pg_constraint r
-        JOIN pkdeps on r.oid = pkdeps.objid
-        JOIN pg_class c on r.conrelid = c.oid
-        JOIN pg_namespace n on c.relnamespace = n.oid
-        JOIN pg_class cf on r.confrelid = cf.oid
-        JOIN pg_namespace nf on cf.relnamespace = nf.oid
-  where NOT EXISTS (select 1 from knownfkeys where oid = r.oid)"
-                                 pkey-oid-list
-                                 (or fkey-oid-list (list -1))))
+         :in (query nil
+                    (format nil
+                            (sql "/pgsql/list-missing-fk-deps.sql")
+                            pkey-oid-list
+                            (or fkey-oid-list (list -1))))
          ;;
          ;; We don't need to reference the main catalog entries for the tables
          ;; here, as the only goal is to be sure to DROP then CREATE again the
@@ -410,12 +315,10 @@ with pkeys(oid) as (
 (defun list-table-oids (table-names)
   "Return an hash table mapping TABLE-NAME to its OID for all table in the
    TABLE-NAMES list. A PostgreSQL connection must be established already."
-  (let ((oidmap (make-hash-table :size (length table-names) :test #'equal)))
+  (let ((oidmap (make-hash-table :size (length table-names) :test #'equal))
+        (sql    (format nil (sql "/pgsql/list-table-oids.sql") table-names)))
     (when table-names
       (loop :for (name oid)
-         :in (pomo:query
-              (format nil
-                      "select n, n::regclass::oid from (values ~{('~a')~^,~}) as t(n)"
-                      table-names))
+         :in (query nil sql)
          :do (setf (gethash name oidmap) oid)))
     oidmap))
