@@ -34,6 +34,11 @@
 	(setf (slot-value source 'transforms)
               (make-list (length (slot-value source 'columns))))))))
 
+(defgeneric concurrency-support (copy concurrency)
+  (:documentation
+   "Returns nil when no concurrency is supported, or a list of copy ojbects
+    prepared to run concurrently."))
+
 (defgeneric map-rows (source &key process-row-fn)
   (:documentation
    "Load data from SOURCE and funcall PROCESS-ROW-FUN for each row found in
@@ -45,11 +50,10 @@
     CSV user-defined field projections to PostgreSQL columns. This function
     returns the pre-processing function, which must be a funcallable object."))
 
-(defgeneric queue-raw-data (source queue)
+(defgeneric queue-raw-data (source queue concurrency)
   (:documentation "Send raw data from the reader to the worker queue."))
 
-(defgeneric format-data-to-copy (source raw-queue formatted-queue
-                                 &optional pre-formatted)
+(defgeneric data-is-preformatted-p (source)
   (:documentation
    "Process raw data from RAW-QUEUE and prepare batches of formatted text to
     send down to PostgreSQL with the COPY protocol in FORMATTED-QUEUE."))
@@ -175,6 +179,11 @@
 
 (defgeneric prepare-pgsql-database (db-copy catalog
                                     &key
+                                      truncate
+                                      create-tables
+                                      create-schemas
+                                      drop-indexes
+                                      set-table-oids
                                       materialize-views
                                       foreign-keys
                                       include-drop)
@@ -185,8 +194,9 @@
 
 (defgeneric complete-pgsql-database (db-copy catalog pkeys
                                      &key
-                                       data-only
                                        foreign-keys
+                                       create-indexes
+                                       create-triggers
                                        reset-sequences)
   (:documentation "Alter load duties for database sources copy support."))
 
