@@ -14,6 +14,7 @@
                                      truncate
                                      create-tables
                                      create-schemas
+                                     drop-indexes
                                      set-table-oids
                                      materialize-views
                                      foreign-keys
@@ -23,17 +24,22 @@
    tables for materialized views.
 
    That function mutates index definitions in ALL-INDEXES."
-  (log-message :notice "~:[~;DROP then ~]CREATE TABLES" include-drop)
+  (log-message :notice "Prepare PostgreSQL database.")
 
   (with-pgsql-transaction (:pgconn (target-db copy))
-    (when create-schemas
-      (with-stats-collection ("Create Schemas" :section :pre
-                                               :use-result-as-read t
-                                               :use-result-as-rows t)
-        (create-schemas catalog :include-drop include-drop)))
+    (setf (catalog-types-without-btree catalog)
+          (list-typenames-without-btree-support))
 
     (if create-tables
         (progn
+          (when create-schemas
+            (with-stats-collection ("Create Schemas" :section :pre
+                                                     :use-result-as-read t
+                                                     :use-result-as-rows t)
+              (create-schemas catalog
+                              :include-drop include-drop
+                              :client-min-messages :error)))
+
           ;; create new SQL types (ENUMs, SETs) if needed and before we
           ;; get to the table definitions that will use them
           (with-stats-collection ("Create SQL Types" :section :pre
@@ -71,12 +77,13 @@
                                                         :use-result-as-rows t)
               (drop-pgsql-fkeys catalog)))
 
-          (with-stats-collection ("Drop Indexes" :section :pre
-                                                 :use-result-as-read t
-                                                 :use-result-as-rows t)
-            ;; we want to error out early in case we can't DROP the
-            ;; index, don't CASCADE
-            (drop-indexes catalog :cascade nil))
+          (when drop-indexes
+            (with-stats-collection ("Drop Indexes" :section :pre
+                                                   :use-result-as-read t
+                                                   :use-result-as-rows t)
+              ;; we want to error out early in case we can't DROP the
+              ;; index, don't CASCADE
+              (drop-indexes catalog :cascade nil)))
 
           (when truncate
             (with-stats-collection ("Truncate" :section :pre
@@ -98,7 +105,12 @@
       (with-stats-collection ("Create MatViews Tables" :section :pre
                                                        :use-result-as-read t
                                                        :use-result-as-rows t)
-        (create-views catalog :include-drop include-drop)))))
+        (create-views catalog
+                      :include-drop include-drop
+                      :client-min-messages :error))))
+
+  ;; log the catalog we just fetched and (maybe) merged
+  (log-message :data "CATALOG: ~s" catalog))
 
 (defmethod cleanup ((copy db-copy) (catalog catalog) &key materialize-views)
   "In case anything wrong happens at `prepare-pgsql-database' step, this
@@ -110,8 +122,8 @@
                                     (catalog catalog)
                                     pkeys
                                     &key
-                                      data-only
                                       foreign-keys
+                                      create-indexes
                                       create-triggers
                                       reset-sequences)
   "After loading the data into PostgreSQL, we can now reset the sequences
@@ -122,46 +134,69 @@
   ;; able to benefit from the indexes. In particular avoid doing that step
   ;; while CREATE INDEX statements are in flight (avoid locking).
   ;;
+  (log-message :notice "Complete PostgreSQL database.")
+
   (when reset-sequences
     (reset-sequences (clone-connection (target-db copy)) catalog))
 
-  (with-pgsql-connection ((clone-connection (target-db copy)))
-    ;;
-    ;; Turn UNIQUE indexes into PRIMARY KEYS now
-    ;;
-    (unless data-only
-      (pgsql-execute-with-timing :post "Primary Keys" pkeys)
+  (handler-case
+      (with-pgsql-transaction (:pgconn (clone-connection (target-db copy)))
+        ;;
+        ;; Turn UNIQUE indexes into PRIMARY KEYS now
+        ;;
+        (when create-indexes
+          (pgsql-execute-with-timing :post "Primary Keys" pkeys)
 
-      ;;
-      ;; Foreign Key Constraints
-      ;;
-      ;; We need to have finished loading both the reference and the refering
-      ;; tables to be able to build the foreign keys, so wait until all tables
-      ;; and indexes are imported before doing that.
-      ;;
-      (when foreign-keys
-        (with-stats-collection ("Create Foreign Keys" :section :post
-                                                      :use-result-as-read t
-                                                      :use-result-as-rows t)
-          (create-pgsql-fkeys catalog)))
+          ;;
+          ;; Foreign Key Constraints
+          ;;
+          ;; We need to have finished loading both the reference and the
+          ;; refering tables to be able to build the foreign keys, so wait
+          ;; until all tables and indexes are imported before doing that.
+          ;;
+          (when foreign-keys
+            (create-pgsql-fkeys catalog
+                                :section :post
+                                :label "Create Foreign Keys"))
 
-      ;;
-      ;; Triggers and stored procedures -- includes special default values
-      ;;
-      (when create-triggers
-        (with-stats-collection ("Create Triggers" :section :post
-                                                  :use-result-as-read t
-                                                  :use-result-as-rows t)
-          (with-pgsql-transaction (:pgconn (target-db copy))
-            (create-triggers catalog)))))
+          ;;
+          ;; Triggers and stored procedures -- includes special default values
+          ;;
+          (when create-triggers
+            (create-triggers catalog
+                             :section :post
+                             :label "Create Triggers")))
 
-    ;;
-    ;; And now, comments on tables and columns.
-    ;;
-    (with-stats-collection ("Install Comments" :section :post
-                                               :use-result-as-read t
-                                               :use-result-as-rows t)
-      (comment-on-tables-and-columns catalog))))
+        ;;
+        ;; And now, comments on tables and columns.
+        ;;
+        (comment-on-tables-and-columns catalog
+                                       :section :post
+                                       :label "Install Comments"))
+
+    (postgresql-unavailable (condition)
+
+      (log-message :error "~a" condition)
+      (log-message :error
+                   "Complete PostgreSQL database reconnecting to PostgreSQL.")
+
+     ;; in order to avoid Socket error in "connect": ECONNREFUSED if we
+     ;; try just too soon, wait a little
+      (sleep 2)
+
+
+     ;;
+     ;; Reset Sequence can be done several times safely, and the rest of the
+     ;; operations run in a single transaction, so if the connection was lost,
+     ;; nothing has been done. Retry.
+     ;;
+      (complete-pgsql-database copy
+                               catalog
+                               pkeys
+                               :foreign-keys foreign-keys
+                               :create-indexes create-indexes
+                               :create-triggers create-triggers
+                               :reset-sequences reset-sequences))))
 
 (defmethod instanciate-table-copy-object ((copy db-copy) (table table))
   "Create an new instance for copying TABLE data."
@@ -204,18 +239,20 @@
                             (on-error-stop    *on-error-stop*)
                             (worker-count     4)
                             (concurrency      1)
+                            (multiple-readers nil)
                             max-parallel-create-index
 			    (truncate         nil)
 			    (disable-triggers nil)
 			    (data-only        nil)
 			    (schema-only      nil)
-                            (create-schemas   nil)
+                            (create-schemas   t)
 			    (create-tables    t)
 			    (include-drop     t)
 			    (create-indexes   t)
                             (index-names      :uniquify)
 			    (reset-sequences  t)
 			    (foreign-keys     t)
+                            (reindex          nil)
 			    only-tables
 			    including
 			    excluding
@@ -224,7 +261,24 @@
                             alter-schema
 			    materialize-views)
   "Export database source data and Import it into PostgreSQL"
-  (let* ((copy-kernel  (make-kernel worker-count))
+  (let* ((*on-error-stop* on-error-stop)
+         (copy-data      (or data-only (not schema-only)))
+         (create-ddl     (or schema-only (not data-only)))
+         (create-tables  (and create-tables create-ddl))
+         (create-schemas (and create-schemas create-ddl))
+         (foreign-keys   (and foreign-keys create-ddl))
+         (drop-indexes   (or reindex
+                             (and include-drop create-ddl)))
+         (create-indexes (or reindex
+                             (and create-indexes drop-indexes create-ddl)))
+
+         (*preserve-index-names*
+          (or (eq :preserve index-names)
+              ;; if we didn't create the tables, we are re-installing the
+              ;; pre-existing indexes
+              (not create-tables)))
+
+         (copy-kernel  (make-kernel worker-count))
          (copy-channel (let ((lp:*kernel* copy-kernel)) (lp:make-channel)))
          (catalog      (fetch-metadata
                         copy
@@ -234,9 +288,9 @@
                                  (fd-connection (pathname-name
                                                  (fd-path (source-db copy))))))
                         :materialize-views materialize-views
-                        :only-tables only-tables
                         :create-indexes create-indexes
                         :foreign-keys foreign-keys
+                        :only-tables only-tables
                         :including including
                         :excluding excluding))
          pkeys
@@ -248,7 +302,9 @@
                                            max-indexes))))
          (idx-channel   (when idx-kernel
                           (let ((lp:*kernel* idx-kernel))
-                            (lp:make-channel)))))
+                            (lp:make-channel))))
+
+         (task-count    0))
 
     ;; apply catalog level transformations to support the database migration
     ;; that's CAST rules, index WHERE clause rewriting and ALTER commands
@@ -260,17 +316,12 @@
     (handler-case
         (prepare-pgsql-database copy
                                 catalog
-                                :truncate (and truncate (not create-tables))
-                                :create-tables (and create-tables
-                                                    (or schema-only
-                                                        (not data-only)))
-                                :create-schemas (and create-schemas
-                                                     (or schema-only
-                                                         (not data-only)))
+                                :truncate truncate
+                                :create-tables create-tables
+                                :create-schemas create-schemas
+                                :drop-indexes drop-indexes
                                 :include-drop include-drop
-                                :foreign-keys (and foreign-keys
-                                                   (or schema-only
-                                                       (not data-only)))
+                                :foreign-keys foreign-keys
                                 :set-table-oids set-table-oids
                                 :materialize-views materialize-views)
       ;;
@@ -295,64 +346,68 @@
 
        :do (let ((table-source (instanciate-table-copy-object copy table)))
              ;; first COPY the data from source to PostgreSQL, using copy-kernel
-             (unless schema-only
-               ;; prepare the writers-count hash-table, as we start
-               ;; copy-from, we have concurrency tasks writing.
-               (setf (gethash table writers-count) concurrency)
-               (copy-from table-source
-                          :concurrency concurrency
-                          :kernel copy-kernel
-                          :channel copy-channel
-                          :on-error-stop on-error-stop
-                          :disable-triggers disable-triggers))))
+             (if (not copy-data)
+                 ;; start indexing straight away then
+                 (when create-indexes
+                   (alexandria:appendf
+                    pkeys
+                    (create-indexes-in-kernel (target-db copy)
+                                              table
+                                              idx-kernel
+                                              idx-channel)))
+
+                 ;; prepare the writers-count hash-table, as we start
+                 ;; copy-from, we have concurrency tasks writing.
+                 (progn                 ; when copy-data
+                   (setf (gethash table writers-count) concurrency)
+
+                   (incf task-count
+                         (copy-from table-source
+                                    :concurrency concurrency
+                                    :multiple-readers multiple-readers
+                                    :kernel copy-kernel
+                                    :channel copy-channel
+                                    :on-error-stop on-error-stop
+                                    :disable-triggers disable-triggers))))))
 
     ;; now end the kernels
     ;; and each time a table is done, launch its indexing
-    (unless schema-only
+    (when copy-data
       (let ((lp:*kernel* copy-kernel))
         (with-stats-collection ("COPY Threads Completion" :section :post
                                                           :use-result-as-read t
                                                           :use-result-as-rows t)
-            (let ((worker-count (* (hash-table-count writers-count)
-                                   (task-count concurrency))))
-              (loop :for tasks :below worker-count
-                 :do (destructuring-bind (task table seconds)
-                         (lp:receive-result copy-channel)
-                       (log-message :debug
-                                    "Finished processing ~a for ~s ~50T~6$s"
-                                    task (format-table-name table) seconds)
-                       (when (eq :writer task)
-                         (update-stats :data table :secs seconds)
+          (loop :repeat task-count
+             :do (destructuring-bind (task table seconds)
+                     (lp:receive-result copy-channel)
+                   (log-message :debug
+                                "Finished processing ~a for ~s ~50T~6$s"
+                                task (format-table-name table) seconds)
+                   (when (eq :writer task)
+                     ;;
+                     ;; Start the CREATE INDEX parallel tasks only when
+                     ;; the data has been fully copied over to the
+                     ;; corresponding table, that's when the writers
+                     ;; count is down to zero.
+                     ;;
+                     (decf (gethash table writers-count))
+                     (log-message :debug "writers-counts[~a] = ~a"
+                                  (format-table-name table)
+                                  (gethash table writers-count))
 
-                         ;;
-                         ;; Start the CREATE INDEX parallel tasks only when
-                         ;; the data has been fully copied over to the
-                         ;; corresponding table, that's when the writers
-                         ;; count is down to zero.
-                         ;;
-                         (decf (gethash table writers-count))
-                         (log-message :debug "writers-counts[~a] = ~a"
-                                      (format-table-name table)
-                                      (gethash table writers-count))
+                     (when (and create-indexes
+                                (zerop (gethash table writers-count)))
+                       (alexandria:appendf
+                        pkeys
+                        (create-indexes-in-kernel (target-db copy)
+                                                  table
+                                                  idx-kernel
+                                                  idx-channel)))))
+             :finally (progn
+                        (lp:end-kernel :wait nil)
+                        (return worker-count))))))
 
-                         (when (and create-indexes
-                                    (not data-only)
-                                    (zerop (gethash table writers-count)))
-                           (let* ((*preserve-index-names*
-                                   (or (eq :preserve index-names)
-                                       ;; if we didn't create the tables, we
-                                       ;; are re-installing the pre-existing
-                                       ;; indexes
-                                       (not create-tables))))
-                             (alexandria:appendf
-                              pkeys
-                              (create-indexes-in-kernel (target-db copy)
-                                                        table
-                                                        idx-kernel
-                                                        idx-channel)))))))
-              (prog1
-                  worker-count
-                (lp:end-kernel :wait nil))))))
+    (log-message :info "Done with COPYing data, waiting for indexes")
 
     (when create-indexes
       (let ((lp:*kernel* idx-kernel))
@@ -361,8 +416,8 @@
         (with-stats-collection ("Index Build Completion" :section :post
                                                          :use-result-as-read t
                                                          :use-result-as-rows t)
-            (loop :for count :below (count-indexes catalog)
-               :do (lp:receive-result idx-channel))
+          (loop :for count :below (count-indexes catalog)
+             :do (lp:receive-result idx-channel))
           (lp:end-kernel :wait t)
           (count-indexes catalog))))
 
@@ -372,8 +427,8 @@
     (complete-pgsql-database copy
                              catalog
                              pkeys
-                             :data-only data-only
                              :foreign-keys foreign-keys
+                             :create-indexes create-indexes
                              ;; only create triggers (for default values)
                              ;; when we've been responsible for creating the
                              ;; tables -- otherwise assume the schema is
