@@ -26,8 +26,8 @@
 	   (field-name-as-symbol (field-name-or-list)
 	     "we need to deal with symbols as we generate code"
 	     (typecase field-name-or-list
-	       (list (pgloader.transforms:intern-symbol (car field-name-or-list)))
-	       (t    (pgloader.transforms:intern-symbol field-name-or-list))))
+	       (list (intern-symbol (car field-name-or-list)))
+	       (t    (intern-symbol field-name-or-list))))
 
 	   (process-field (field-name-or-list)
 	     "Given a field entry, return a function dealing with nulls for it"
@@ -107,8 +107,7 @@
 			  ;; we expect the name of a COLUMN to be the same
 			  ;; as the name of its derived FIELD when we
 			  ;; don't have any transformation function
-			    (or fn `(funcall ,(process-field name)
-					     ,(field-name-as-symbol name))))))
+                            (or fn (field-name-as-symbol name)))))
 		 `(lambda (row)
 		    (declare (optimize speed) (type list row))
 		    (destructuring-bind (&optional ,@args &rest extra) row
@@ -119,29 +118,21 @@
       ;; allow for some debugging
       (if compile (compile nil projection) projection))))
 
-(defun reformat-then-process (&key fields columns target process-row-fn)
+(defun reformat-then-process (&key fields columns target)
   "Return a lambda form to apply to each row we read.
 
    The lambda closes over the READ paramater, which is a counter of how many
    lines we did read in the file."
   (let ((projection (project-fields :fields fields :columns columns)))
     (lambda (row)
-      (pgstate-incf *state* target :read 1)
       ;; cl-csv returns (nil) for an empty line
       (if (or (null row)
               (and (null (car row)) (null (cdr row))))
-          (log-message :notice "Skipping empty line ~d."
-                       (pgloader.utils::pgtable-read
-                        (pgstate-get-table *state* target)))
-          (let ((projected-vector
-                 (handler-case
-                     (funcall projection row)
-                   (condition (e)
-                     (pgstate-incf *state* target :errs 1)
-                     (log-message :error "Could not read line ~d: ~a"
-                                  (pgloader.utils::pgtable-read
-                                   (pgstate-get-table *state* target))
-                                  e)))))
-            (when projected-vector
-              (funcall process-row-fn projected-vector)))))))
+          (log-message :notice "Skipping empty line.")
+
+          (handler-case
+              (funcall projection row)
+            (condition (e)
+              (update-stats :data target :errs 1)
+              (log-message :error "Could not read input: ~a" e)))))))
 
