@@ -135,6 +135,10 @@
      :using pgloader.transforms::set-to-enum-array)
 
     ;; geometric data types, just POINT for now
+    (:source (:type "geometry")
+     :target (:type "point")
+     :using pgloader.transforms::convert-mysql-point)
+
     (:source (:type "point")
      :target (:type "point")
      :using pgloader.transforms::convert-mysql-point))
@@ -171,7 +175,7 @@
                         (or (uiop:string-prefix-p "CURRENT_TIMESTAMP" default)
                             (string= "CURRENT TIMESTAMP" default)))
                    :current-timestamp)
-                  (t default)))
+                  (t (column-default pgcol))))
 
       ;; extra user-defined data types
       (when (or (string-equal "set" dtype)
