@@ -29,6 +29,11 @@
                                  :password (db-pass myconn)
                                  :database (db-name myconn))))
   (log-message :debug "CONNECTED TO ~a" myconn)
+
+  ;; apply mysql-settings, if any
+  (loop :for (name . value) :in *mysql-settings*
+     :for sql := (format nil "set ~a = ~a;" name value)
+     :do (query myconn sql))
   ;; return the connection object
   myconn)
 
@@ -47,7 +52,7 @@
                     (as-text t)
                     (result-type 'list))
   "Run SQL query against MySQL connection MYCONN."
-  (log-message :debug "MySQL: sending query: ~a" sql)
+  (log-message :sql "MySQL: sending query: ~a" sql)
   (qmynd:mysql-query (conn-handle myconn)
                      sql
                      :row-fn row-fn
@@ -57,14 +62,14 @@
 ;;;
 ;;; The generic API query is recent, used to look like this:
 ;;;
+(declaim (inline mysql-query))
 (defun mysql-query (query &key row-fn (as-text t) (result-type 'list))
   "Execute given QUERY within the current *connection*, and set proper
    defaults for pgloader."
-  (log-message :debug "MySQL: sending query: ~a" query)
-  (qmynd:mysql-query (conn-handle *connection*) query
-                     :row-fn row-fn
-                     :as-text as-text
-                     :result-type result-type))
+  (query *connection* query
+         :row-fn row-fn
+         :as-text as-text
+         :result-type result-type))
 
 ;;;
 ;;; Function for accessing the MySQL catalogs, implementing auto-discovery.
@@ -176,18 +181,8 @@
   (loop
      :for (tname tcomment cname ccomment dtype ctype default nullable extra)
      :in
-     (mysql-query (format nil "
-  select c.table_name, t.table_comment,
-         c.column_name, c.column_comment,
-         c.data_type, c.column_type, c.column_default,
-         c.is_nullable, c.extra
-    from information_schema.columns c
-         join information_schema.tables t using(table_schema, table_name)
-   where c.table_schema = '~a' and t.table_type = '~a'
-         ~:[~*~;and table_name in (~{'~a'~^,~})~]
-         ~:[~*~;and (~{table_name ~a~^ or ~})~]
-         ~:[~*~;and (~{table_name ~a~^ and ~})~]
-order by table_name, ordinal_position"
+     (mysql-query (format nil
+                          (sql "/mysql/list-all-columns.sql")
                           (db-name *connection*)
                           table-type-name
                           only-tables   ; do we print the clause?
@@ -219,15 +214,8 @@ order by table_name, ordinal_position"
   "Get the list of MySQL index definitions per table."
   (loop
      :for (table-name name non-unique cols)
-     :in (mysql-query (format nil "
-  SELECT table_name, index_name, sum(non_unique),
-         cast(GROUP_CONCAT(column_name order by seq_in_index) as char)
-    FROM information_schema.statistics
-   WHERE table_schema = '~a'
-         ~:[~*~;and table_name in (~{'~a'~^,~})~]
-         ~:[~*~;and (~{table_name ~a~^ or ~})~]
-         ~:[~*~;and (~{table_name ~a~^ and ~})~]
-GROUP BY table_name, index_name;"
+     :in (mysql-query (format nil
+                              (sql "/mysql/list-all-indexes.sql")
                               (db-name *connection*)
                               only-tables ; do we print the clause?
                               only-tables
@@ -260,41 +248,8 @@ GROUP BY table_name, index_name;"
   "Get the list of MySQL Foreign Keys definitions per table."
   (loop
      :for (table-name name ftable-name cols fcols update-rule delete-rule)
-     :in (mysql-query (format nil "
-SELECT s.table_name, s.constraint_name, s.ft, s.cols, s.fcols,
-       rc.update_rule, rc.delete_rule
-
-FROM
- (
-  SELECT tc.table_schema, tc.table_name,
-         tc.constraint_name, k.referenced_table_name ft,
-
-             group_concat(         k.column_name
-                          order by k.ordinal_position) as cols,
-
-             group_concat(         k.referenced_column_name
-                          order by k.position_in_unique_constraint) as fcols
-
-        FROM information_schema.table_constraints tc
-
-        LEFT JOIN information_schema.key_column_usage k
-               ON k.table_schema = tc.table_schema
-              AND k.table_name = tc.table_name
-              AND k.constraint_name = tc.constraint_name
-
-      WHERE     tc.table_schema = '~a'
-            AND k.referenced_table_schema = '~a'
-            AND tc.constraint_type = 'FOREIGN KEY'
-           ~:[~*~;and tc.table_name in (~{'~a'~^,~})~]
-           ~:[~*~;and (~{tc.table_name ~a~^ or ~})~]
-           ~:[~*~;and (~{tc.table_name ~a~^ and ~})~]
-
-   GROUP BY tc.table_schema, tc.table_name, tc.constraint_name, ft
- ) s
-             JOIN information_schema.referential_constraints rc
-               ON rc.constraint_schema = s.table_schema
-              AND rc.constraint_name = s.constraint_name
-              AND rc.table_name = s.table_name"
+     :in (mysql-query (format nil
+                              (sql "/mysql/list-all-fkeys.sql")
                               (db-name *connection*) (db-name *connection*)
                               only-tables ; do we print the clause?
                               only-tables
@@ -342,14 +297,8 @@ FROM
   "Return comments on MySQL tables."
   (loop
      :for (table-name comment)
-     :in (mysql-query (format nil "
-    SELECT table_name, table_comment
-      FROM information_schema.tables
-    WHERE     table_schema = '~a'
-          and table_type = 'BASE TABLE'
-         ~:[~*~;and table_name in (~{'~a'~^,~})~]
-         ~:[~*~;and (~{table_name ~a~^ or ~})~]
-         ~:[~*~;and (~{table_name ~a~^ and ~})~]"
+     :in (mysql-query (format nil
+                              (sql "/mysql/list-table-comments.sql")
                               (db-name *connection*)
                               only-tables ; do we print the clause?
                               only-tables
@@ -367,16 +316,8 @@ FROM
   "Return comments on MySQL tables."
   (loop
      :for (table-name column-name comment)
-     :in (mysql-query (format nil "
-  select c.table_name, c.column_name, c.column_comment
-    from information_schema.columns c
-         join information_schema.tables t using(table_schema, table_name)
-   where     c.table_schema = '~a'
-         and t.table_type = 'BASE TABLE'
-         ~:[~*~;and table_name in (~{'~a'~^,~})~]
-         ~:[~*~;and (~{table_name ~a~^ or ~})~]
-         ~:[~*~;and (~{table_name ~a~^ and ~})~]
-order by table_name, ordinal_position"
+     :in (mysql-query (format nil
+                              (sql "/mysql/list-columns-comments.sql")
                               (db-name *connection*)
                               only-tables ; do we print the clause?
                               only-tables
@@ -398,8 +339,9 @@ order by table_name, ordinal_position"
    Mostly we just use the name, but in case of POINT we need to use
    astext(name)."
   (case (intern (string-upcase type) "KEYWORD")
-    (:point  (format nil "astext(`~a`) as `~a`" name name))
-    (t       (format nil "`~a`" name))))
+    (:geometry  (format nil "astext(`~a`) as `~a`" name name))
+    (:point     (format nil "astext(`~a`) as `~a`" name name))
+    (t          (format nil "`~a`" name))))
 
 (defun get-column-list (dbname table-name)
   "Some MySQL datatypes have a meaningless default output representation, we
@@ -407,13 +349,10 @@ order by table_name, ordinal_position"
 
    This function assumes a valid connection to the MySQL server has been
    established already."
-  (loop
-     for (name type) in (mysql-query (format nil "
-  select column_name, data_type
-    from information_schema.columns
-   where table_schema = '~a' and table_name = '~a'
-order by ordinal_position" dbname table-name)
-					   :result-type 'list)
+  (loop with sql = (format nil
+                           (sql "/mysql/get-column-list.sql")
+                           dbname table-name)
+     for (name type) in (mysql-query sql :result-type 'list)
      collect (get-column-sql-expression name type)))
 
 (declaim (inline fix-nulls))
