@@ -9,7 +9,8 @@
 (defcategory :error   (or :error :log))
 (defcategory :warning (or :warning :error))
 (defcategory :notice  (or :notice :warning))
-(defcategory :info    (or :info :notice))
+(defcategory :sql     (or :sql :notice))
+(defcategory :info    (or :info :sql))
 (defcategory :debug   (or :debug :info))
 (defcategory :data    (or :data :debug))
 
