@@ -10,14 +10,17 @@
 			  `((*monitoring-queue*   . ,*monitoring-queue*)
                             (*copy-batch-rows*    . ,*copy-batch-rows*)
                             (*copy-batch-size*    . ,*copy-batch-size*)
-                            (*concurrent-batches* . ,*concurrent-batches*)
+                            (*rows-per-range*     . ,*rows-per-range*)
+                            (*prefetch-rows*      . ,*prefetch-rows*)
 			    (*pg-settings*        . ',*pg-settings*)
-			    (*state*              . ,*state*)
+			    (*mysql-settings*     . ',*mysql-settings*)
+                            (*root-dir*           . ,*root-dir*)
                             (*fd-path-root*       . ,*fd-path-root*)
                             (*client-min-messages* . ,*client-min-messages*)
                             (*log-min-messages*    . ,*log-min-messages*)
 
                             ;; needed in create index specific kernels
+                            (*pgsql-reserved-keywords* . ',*pgsql-reserved-keywords*)
                             (*preserve-index-names* . ,*preserve-index-names*)
 
                             ;; bindings updates for libs
