@@ -7,6 +7,52 @@
 
 (in-package :pgloader.transforms)
 
+;;;
+;;; This package is used to generate symbols in the CL code that
+;;; project-fields produces, allowing to use symbols such as t. It's
+;;; important that the user-symbols package doesn't :use cl.
+;;;
+(defpackage #:pgloader.user-symbols (:use))
+
+(defun intern-symbol (symbol-name &optional (overrides '()))
+  "Return a symbol in either PGLOADER.TRANSFORMS if it exists there
+   already (it's a user provided function) or a PGLOADER.USER-SYMBOLS
+   package.
+
+   OVERRIDES is an alist of symbol . value, allowing called to force certain
+   values: the classic example is how to parse the \"nil\" symbol-name.
+   Given OVERRIDES as '((nil . nil)) the returned symbol will be cl:nil
+   rather than pgloader.user-symbols::nil."
+  (let ((overriden (assoc symbol-name overrides :test #'string-equal)))
+    (if overriden
+        (cdr overriden)
+
+        (multiple-value-bind (symbol status)
+            (find-symbol (string-upcase symbol-name)
+                         (find-package "PGLOADER.TRANSFORMS"))
+          ;; pgloader.transforms package (:use :cl) so we might find variable
+          ;; names in there that we want to actually intern in
+          ;; pgloader.user-symbols so that users may use e.g. t as a column name...
+          ;; so only use transform symbol when it denotes a function
+          (cond
+            ((and status (fboundp symbol)) symbol) ; a transform function
+
+            (t
+             (intern (string-upcase symbol-name)
+                     (find-package "PGLOADER.USER-SYMBOLS"))))))))
+
+(defun typemod-expr-to-function (expr)
+  "Transform given EXPR into a callable function object."
+  `(lambda (typemod)
+     (destructuring-bind (precision &optional (scale 0))
+         typemod
+       (declare (ignorable precision scale))
+       ,expr)))
+
+
+;;;
+;;; Some optimisation stanza
+;;;
 (declaim (inline intern-symbol
 		 zero-dates-to-null
 		 date-with-no-separator
@@ -16,22 +62,16 @@
 		 int-to-ip
 		 ip-range
 		 convert-mysql-point
-		 float-to-string
+		 integer-to-string
+                 float-to-string
                  empty-string-to-null
 		 set-to-enum-array
 		 right-trim
 		 byte-vector-to-bytea
                  sqlite-timestamp-to-timestamp
                  sql-server-uniqueidentifier-to-uuid
-                 sql-server-bit-to-boolean))
-
-
-;;;
-;;; Some tools for reading expressions in the parser, and evaluating them.
-;;;
-(defun intern-symbol (symbol-name)
-  (intern (string-upcase symbol-name)
-	  (find-package "PGLOADER.TRANSFORMS")))
+                 sql-server-bit-to-boolean
+                 varbinary-to-string))
 
 
 ;;;
@@ -147,6 +187,22 @@
       (setf (aref point (position #\Space point)) #\,)
       point)))
 
+(defun integer-to-string (integer-string)
+  "Transform INTEGER-STRING parameter into a proper string representation of
+   it. In particular be careful of quoted-integers, thanks to SQLite default
+   values."
+  (declare (type (or null string fixnum) integer-string))
+  (when integer-string
+    (princ-to-string
+     (typecase integer-string
+       (integer integer-string)
+       (string  (handler-case
+                    (parse-integer integer-string :start 0)
+                  (condition (c)
+                    (declare (ignore c))
+                    (parse-integer integer-string :start 1
+                                   :end (- (length integer-string) 1)))))))))
+
 (defun float-to-string (float)
   "Transform a Common Lisp float value into its string representation as
    accepted by PostgreSQL, that is 100.0 rather than 100.0d0."
@@ -174,31 +230,41 @@
   (when string
     (string-right-trim '(#\Space) string)))
 
+(defun remove-null-characters (string)
+  "Remove NULL-characters (0x00) from STRING"
+  (when string
+    (remove #\Nul string)))
+
 (defun byte-vector-to-bytea (vector)
   "Transform a simple array of unsigned bytes to the PostgreSQL bytea
   representation as documented at
   http://www.postgresql.org/docs/9.3/interactive/datatype-binary.html
 
   Note that we choose here the bytea Hex Format."
-  (declare (type (or null (simple-array (unsigned-byte 8) (*))) vector))
-  (when vector
-    (let ((hex-digits "0123456789abcdef")
-	  (bytea (make-array (+ 2 (* 2 (length vector)))
-			     :initial-element #\0
-			     :element-type 'standard-char)))
+  (declare (type (or null string (simple-array (unsigned-byte 8) (*))) vector))
+  (etypecase vector
+    (null nil)
+    (string (if (string= "" vector)
+                nil
+                (error "byte-vector-to-bytea called on a string: ~s" vector)))
+    (simple-array
+     (let ((hex-digits "0123456789abcdef")
+           (bytea (make-array (+ 2 (* 2 (length vector)))
+                              :initial-element #\0
+                              :element-type 'standard-char)))
 
-      ;; The entire string is preceded by the sequence \x (to distinguish it
-      ;; from the escape format).
-      (setf (aref bytea 0) #\\)
-      (setf (aref bytea 1) #\x)
+       ;; The entire string is preceded by the sequence \x (to distinguish it
+       ;; from the escape format).
+       (setf (aref bytea 0) #\\)
+       (setf (aref bytea 1) #\x)
 
-      (loop for pos from 2 by 2
-	 for byte across vector
-	 do (let ((high (ldb (byte 4 4) byte))
-		  (low  (ldb (byte 4 0) byte)))
-	      (setf (aref bytea pos)       (aref hex-digits high))
-	      (setf (aref bytea (+ pos 1)) (aref hex-digits low)))
-	 finally (return bytea)))))
+       (loop for pos from 2 by 2
+          for byte across vector
+          do (let ((high (ldb (byte 4 4) byte))
+                   (low  (ldb (byte 4 0) byte)))
+               (setf (aref bytea pos)       (aref hex-digits high))
+               (setf (aref bytea (+ pos 1)) (aref hex-digits low)))
+          finally (return bytea))))))
 
 (defun ensure-parse-integer (string-or-integer)
   "Return an integer value if string-or-integer is an integer or a string
@@ -266,3 +332,10 @@
            ((string= "((1))" bit-string-or-integer) "t")
            (t nil)))))
 
+(defun varbinary-to-string (string)
+  (let ((babel::*default-character-encoding*
+         (or qmynd::*mysql-encoding*
+             babel::*default-character-encoding*)))
+    (etypecase string
+      (string string)
+      (vector (babel:octets-to-string string)))))
