@@ -62,3 +62,35 @@
 
            (t
             (uiop:parse-unix-namestring namestring))))))
+
+;;;
+;;; For log messages
+;;;
+(defun pretty-print-bytes (bytes &key (unit "B"))
+  "Return a string to reprensent bytes in human readable format, with units"
+  (let ((bytes (or bytes 0)))
+    (loop
+       :for multiple :in '("T" "G" "M" "k")
+       :for power :in '(40 30 20 10 1)
+       :for limit := (expt 2 power)
+       :until (<= limit bytes)
+       :finally (return
+                  (format nil "~5,1f ~a~a" (/ bytes limit) multiple unit)))))
+
+;;;
+;;; Defining ranges and partitions.
+;;;
+(defun split-range (min max &optional (count *rows-per-range*))
+  "Split the range from MIN to MAX into sub-ranges of COUNT elements."
+  (loop :for i := min :then j
+     :for j := (+ i count)
+     :while (< i max)
+     :collect (list i (min j max))))
+
+(defun distribute (list-of-ranges count)
+  "Split a list of ranges into COUNT sublists."
+  (let ((result (make-array count :element-type 'list :initial-element nil)))
+    (loop :for i :from 0
+       :for range :in list-of-ranges
+       :do (push range (aref result (mod i count))))
+    (map 'list #'reverse result )))
